"""RWTH-PHOENIX 2014 T: Parallel Corpus of Sign Language Video, Gloss and Translation"""
import csv
from os import path

import tensorflow as tf
import tensorflow_datasets as tfds
from tensorflow.io.gfile import GFile

from ..warning import dataset_warning
from ...datasets.config import SignDatasetConfig
from ...utils.features import PoseFeature

_DESCRIPTION = """
Parallel Corpus of German Sign Language of the weather, including video, gloss and translation.
Additional poses extracted by MediaPipe Holistic are also available.
"""

_CITATION = """
@inproceedings{cihan2018neural,
  title={Neural sign language translation},
  author={Cihan Camgoz, Necati and Hadfield, Simon and Koller, Oscar and Ney, Hermann and Bowden, Richard},
  booktitle={Proceedings of the IEEE Conference on Computer Vision and Pattern Recognition},
  pages={7784--7793},
  year={2018}
}
@article{koller2015continuous,
  title={Continuous sign language recognition: 
  Towards large vocabulary statistical recognition systems handling multiple signers},
  author={Koller, Oscar and Forster, Jens and Ney, Hermann},
  journal={Computer Vision and Image Understanding},
  volume={141},
  pages={108--125},
  year={2015},
  publisher={Elsevier}
}
"""

_VIDEO_ANNOTATIONS_URL = "https://www-i6.informatik.rwth-aachen.de/ftp/pub/rwth-phoenix/2016/phoenix-2014-T.v3.tar.gz"
_ANNOTATIONS_URL = "https://nlp.biu.ac.il/~amit/datasets/public/phoenix-annotations.tar.gz"

_POSE_URLS = {"holistic": "https://nlp.biu.ac.il/~amit/datasets/poses/holistic/phoenix.tar.gz"}
_POSE_HEADERS = {"holistic": path.join(path.dirname(path.realpath(__file__)), "pose.header")}


class RWTHPhoenix2014T(tfds.core.GeneratorBasedBuilder):
    """DatasetBuilder for rwth_phoenix_2014_t dataset."""

    VERSION = tfds.core.Version("3.0.0")
    RELEASE_NOTES = {"3.0.0": "Initial release."}

    BUILDER_CONFIGS = [
        SignDatasetConfig(name="default", include_video=True, include_pose="holistic"),
        SignDatasetConfig(name="videos", include_video=True, include_pose=None),
        SignDatasetConfig(name="poses", include_video=False, include_pose="holistic"),
        SignDatasetConfig(name="annotations", include_video=False, include_pose=None),
    ]

    def _info(self) -> tfds.core.DatasetInfo:
        """Returns the dataset metadata."""

        features = {
            "id": tfds.features.Text(),
            "signer": tfds.features.Text(),
            "gloss": tfds.features.Text(),
            "text": tfds.features.Text(),
        }

        if self._builder_config.include_video:
            features["fps"] = tf.int32
            features["video"] = self._builder_config.video_feature((210, 260))

        if self._builder_config.include_pose == "holistic":
            pose_header_path = _POSE_HEADERS[self._builder_config.include_pose]
            stride = 1 if self._builder_config.fps is None else 25 / self._builder_config.fps
            features["pose"] = PoseFeature(shape=(None, 1, 543, 3), header_path=pose_header_path, stride=stride)

        return tfds.core.DatasetInfo(
            builder=self,
            description=_DESCRIPTION,
            features=tfds.features.FeaturesDict(features),
            homepage="https://www-i6.informatik.rwth-aachen.de/~koller/RWTH-PHOENIX-2014-T/",
            supervised_keys=None,
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager: tfds.download.DownloadManager):
        """Returns SplitGenerators."""
        dataset_warning(self)

        urls = [_VIDEO_ANNOTATIONS_URL if self._builder_config.include_video else _ANNOTATIONS_URL]

        if self._builder_config.include_pose is not None:
            urls.append(_POSE_URLS[self._builder_config.include_pose])

        downloads = dl_manager.download_and_extract(urls)
        annotations_path = path.join(downloads[0], "PHOENIX-2014-T-release-v3", "PHOENIX-2014-T")

        if self._builder_config.include_pose == "holistic":
            pose_path = path.join(downloads[1], "holistic")
        else:
            pose_path = None

        return [
            tfds.core.SplitGenerator(
                name=tfds.Split.VALIDATION, gen_kwargs={"annotations_path": annotations_path, "pose_path": pose_path, "split": "dev"},
            ),
            tfds.core.SplitGenerator(
                name=tfds.Split.TEST, gen_kwargs={"annotations_path": annotations_path, "pose_path": pose_path, "split": "test"},
            ),
            tfds.core.SplitGenerator(
                name=tfds.Split.TRAIN, gen_kwargs={"annotations_path": annotations_path, "pose_path": pose_path, "split": "train"},
            ),
        ]

    def _generate_examples(self, annotations_path: str, pose_path: str, split: str):
        """ Yields examples. """

        filepath = path.join(annotations_path, "annotations", "manual", "PHOENIX-2014-T." + split + ".corpus.csv")
        images_path = path.join(annotations_path, "features", "fullFrame-210x260px", split)
        poses_path = path.join(pose_path, split) if pose_path is not None else None

        with GFile(filepath, "r") as f:
            data = csv.DictReader(f, delimiter="|", quoting=csv.QUOTE_NONE)
            for row in data:
                datum = {
                    "id": row["name"],
                    "signer": row["speaker"],
                    "gloss": row["orth"],
                    "text": row["translation"],
                }

                if self._builder_config.include_video:
                    frames_base = path.join(images_path, row["video"])[:-7]
                    datum["video"] = [
                        path.join(frames_base, name)
                        for name in sorted(tf.io.gfile.listdir(frames_base))
                        if name != "createDnnTrainingLabels-profile.py.lprof"
                    ]
                    datum["fps"] = self._builder_config.fps if self._builder_config.fps is not None else 25

                if poses_path is not None:
                    datum["pose"] = path.join(poses_path, datum["id"] + ".pose")

                yield datum["id"], datum



"""Public DGS Corpus: parallel corpus for German Sign Language (DGS) with German and English annotations"""

import re
from collections import defaultdict

import tensorflow as tf
import tensorflow_datasets as tfds

from os import path

from pose_format import Pose

from sign_language_datasets.utils.features import PoseFeature
from ..warning import dataset_warning

from ...datasets.config import SignDatasetConfig

_DESCRIPTION = """
Parallel corpus for German Sign Language (DGS) with German and English annotations
"""

_CITATION = """
@misc{dgscorpus_3,
  title = {MEINE DGS -- annotiert. {\"O}ffentliches Korpus der Deutschen Geb{\"a}rdensprache, 3. Release / MY DGS -- annotated. Public Corpus of German Sign Language, 3rd release},
  author = {Konrad, Reiner and Hanke, Thomas and Langer, Gabriele and Blanck, Dolly and Bleicken, Julian and Hofmann, Ilona and Jezziorski, Olga and K{\"o}nig, Lutz and K{\"o}nig, Susanne and Nishio, Rie and Regen, Anja and Salden, Uta and Wagner, Sven and Worseck, Satu and B{\"o}se, Oliver and Jahn, Elena and Schulder, Marc},
  year = {2020},
  type = {languageresource},
  version = {3.0},
  publisher = {Universit{\"a}t Hamburg},
  url = {https://doi.org/10.25592/dgs.corpus-3.0},
  doi = {10.25592/dgs.corpus-3.0}
}
"""

_HOMEPAGE = "https://www.sign-lang.uni-hamburg.de/meinedgs/ling/types_de.html"

_POSE_HEADERS = {
    "holistic": path.join(path.dirname(path.realpath(__file__)), "holistic.poseheader"),
}

_POSE_URLS = {"holistic": "https://nlp.biu.ac.il/~amit/datasets/poses/holistic/dgs_types/"}


class DgsTypes(tfds.core.GeneratorBasedBuilder):
    """DatasetBuilder for dgs_types dataset."""

    VERSION = tfds.core.Version("3.0.0")
    RELEASE_NOTES = {
        "3.0.0": "3rd release",
    }

    BUILDER_CONFIGS = [
        SignDatasetConfig(name="default", include_video=True, include_pose=None),
        SignDatasetConfig(name="holistic", include_video=False, include_pose="holistic"),
        SignDatasetConfig(name="annotations", include_video=False, include_pose=None),
    ]

    def _info(self) -> tfds.core.DatasetInfo:
        """Returns the dataset metadata."""

        if self._builder_config.process_video:
            video_feature = {"name": tfds.features.Text(), "video": self._builder_config.video_feature((640, 360))}
        else:
            video_feature = {"name": tfds.features.Text(), "video": tfds.features.Text()}

        # Add poses if requested
        if self._builder_config.include_pose == "holistic":
            pose_header_path = _POSE_HEADERS[self._builder_config.include_pose]
            stride = 1 if self._builder_config.fps is None else 25 / self._builder_config.fps
            pose_shape = (None, 1, 576, 3)
            video_feature["pose"] = PoseFeature(shape=pose_shape, stride=stride, header_path=pose_header_path)

        features = {
            "id": tfds.features.Text(),
            "glosses": tfds.features.Sequence(tfds.features.Text()),
            "frequencies": tfds.features.Sequence(tf.int32),
            "hamnosys": tfds.features.Text(),
            "views": tfds.features.Sequence(video_feature),
        }

        return tfds.core.DatasetInfo(
            builder=self,
            description=_DESCRIPTION,
            features=tfds.features.FeaturesDict(features),
            homepage=_HOMEPAGE,
            supervised_keys=None,
            citation=_CITATION,
        )

    def get_galex_data(self, dl_manager: tfds.download.DownloadManager):
        GALEX = "https://www.sign-lang.uni-hamburg.de/galex/"
        index_urls = [f"{GALEX}tystatus/x{i}.html" for i in [2, 3, 4]]
        gloss_urls = set()
        for p in dl_manager.download(index_urls):
            with open(p, "r", encoding="utf-8") as f:
                for match in re.finditer(r"<p class=\"XREF\"><a href=\"\.\.\/(.*?)\"", f.read()):
                    match_url = match.group(1)
                    gloss_urls.add(GALEX + match_url)

        video_urls = {}
        data = []
        for p in dl_manager.download(list(gloss_urls)):
            with open(p, "r", encoding="utf-8") as f:
                content = f.read()
                gloss = re.findall(r"span class=\"Gloss\">(.*?)<", content)[0]
                video = re.findall(r"source src=\"\.\.\/(.*?)\"", content)[0]
                datum = {
                    "id": "galex_" + gloss,
                    "glosses": [gloss],
                    "frequencies": [],
                    "hamnosys": re.findall(r"a class=\"ham\".*?>(.*?)<", content)[0],
                    "views": [{"name": "front", "video": video}],
                }
                data.append(datum)
                video_urls[video] = GALEX + video

        video_paths = dl_manager.download(video_urls) if self.builder_config.include_video else video_urls
        for datum in data:
            for view in datum["views"]:
                view["video"] = video_paths[view["video"]]

        return data

    def get_dgs_data(self, dl_manager: tfds.download.DownloadManager):
        MEINE_DGS = "https://www.sign-lang.uni-hamburg.de/meinedgs/"
        dgs_index = dl_manager.download(MEINE_DGS + "ling/types_de.html")

        gloss_map = defaultdict(list)
        gloss_frequencies = defaultdict(list)

        with open(dgs_index, "r", encoding="utf-8") as f:
            for match in re.finditer(r"<p>(.*?) \((\d+) Tokens?\)( → )?(.*?)</p>", f.read()):
                gloss_id = re.findall(r"\.\.\/types\/(.*?)\.html", match.group(0))[0]
                gloss_frequency = int(match.group(2))
                gloss_frequencies[gloss_id].append(gloss_frequency)
                gloss_text = match.group(1) if match.group(3) is not None else re.findall(r">(.*?)<", match.group(1))[0]
                gloss_map[gloss_id].append(gloss_text)

        gloss_ids = list(gloss_map.keys())
        gloss_urls = {k: MEINE_DGS + "types/" + k + ".html" for k in gloss_ids}
        gloss_paths = dl_manager.download(gloss_urls)

        data = []
        video_urls = {}
        for gloss_id, glosses in gloss_map.items():
            with open(gloss_paths[gloss_id], "r", encoding="utf-8") as f:
                content = f.read()

            views = []

            video_src = re.findall(r"<source src=\"(.*?)\"", content)
            if len(video_src) > 0:
                full_src = video_src[0]
                views_info = re.findall(r"class=\"perspectives\".*?\'_(\d)\'.*?>(.*?)<", content)
                for view_id, view_name in views_info:
                    view_video_url = full_src.replace("_1.mp4", f"_{view_id}.mp4")
                    views.append({"name": view_name, "video": view_video_url})
                    video_urls[view_video_url] = view_video_url

            frequencies = gloss_frequencies[gloss_id]

            hamnosys_search = re.findall(r"class=\"hamnosys\".*?>(.*?)<", content)
            hamnosys = hamnosys_search[0] if len(hamnosys_search) > 0 else ""

            data.append(
                {"id": gloss_id, "frequencies": frequencies, "glosses": glosses, "hamnosys": hamnosys, "views": views})

        if self.builder_config.include_video:
            video_paths = dl_manager.download(video_urls)
            for datum in data:
                for view in datum["views"]:
                    view["video"] = video_paths[view["video"]]

        return data

    def _split_generators(self, dl_manager: tfds.download.DownloadManager):
        """Returns SplitGenerators."""
        dataset_warning(self)

        # Source 1: GALEX
        galex_data = self.get_galex_data(dl_manager)

        # Source 2: Meine DGS Types
        dgs_data = self.get_dgs_data(dl_manager)

        data = galex_data + dgs_data

        poses = {}

        # Poses
        if self.builder_config.include_pose == "holistic":
            pose_urls = {
                f"{datum['id']}_{view['name']}": _POSE_URLS["holistic"] + f"{datum['id']}_{view['name']}.pose"
                for datum in data for view in datum["views"]}
            poses = dl_manager.download(pose_urls)

        return [tfds.core.SplitGenerator(name=tfds.Split.TRAIN, gen_kwargs={"data": data, "poses": poses})]

    def _generate_examples(self, data, poses):
        """ Yields examples. """

        for datum in data:
            for view in datum["views"]:
                # convert PosixGPath to str
                view["video"] = str(view["video"])

                if self._builder_config.include_pose:

                    pose_view = f"{datum['id']}_{view['name']}"
                    if pose_view in poses:
                        view["pose"] = poses[pose_view]
                    else:
                        print(f"No pose for {pose_view}.")
                        view["pose"] = None

            yield datum["id"], datum



import json
import re

import tensorflow_datasets as tfds
import hashlib

# noinspection PyUnresolvedReferences
import sign_language_datasets.datasets
from sign_language_datasets.datasets.config import SignDatasetConfig


def clear_gloss(gloss: str):
    text = re.search(r'^(.*?)(?=\d|$)', gloss)
    word = text.group(1)
    return word[0].upper() + word[1:].lower()


def dgs_types():
    config = SignDatasetConfig(name="firestore", version="1.0.0", include_video=False)
    dataset = tfds.load(name='dgs_types', builder_kwargs={"config": config})

    for datum in dataset["train"]:
        uid_raw = "dgs_types_" + datum['id'].numpy().decode('utf-8')

        glosses = [clear_gloss(g.numpy().decode('utf-8')) for g in datum['glosses']]
        captions = [{"language": "de", "transcription": g} for g in glosses]
        captions.append({"language": "hns", "transcription": datum['hamnosys'].numpy().decode('utf-8')})

        views = {view.numpy().decode('utf-8'): {
            "video": video.numpy().decode('utf-8'),
            "pose": None
        } for view, video in zip(datum["views"]["name"], datum["views"]["video"])}

        id_func = lambda view_name: 'dgstypes' + hashlib.md5((uid_raw + view_name).encode()).hexdigest()
        ids = [id_func(view_name) for view_name in views.keys()]

        for view_name, view_data in views.items():
            this_id = id_func(view_name)
            doc = {
                "uid": this_id,
                "url": view_data["video"],
                "meta": {
                    "visibility": "unlisted",
                    "name": f"DGS Types: {' / '.join(glosses)} ({view_name})",
                    "language": 'gsg',  # German Sign Language
                    "userId": "dxnkijRxuTP5JEQMaJu88Cxxlu52"  # DGS Account
                },
                "related": [_id for _id in ids if _id != this_id],
                "context": {
                    "id": uid_raw
                }
            }

            yield doc, captions, None


if __name__ == '__main__':
    with open('dgs_types.jsonl', 'w') as f:
        for doc, captions, _ in dgs_types():
            f.write(json.dumps({"doc": doc, "captions": captions}))
            f.write('\n')



import pympi

"""
Notes:

Annotation guidelines of third release: https://archive.mpi.nl/tla/islandora/object/tla%3A1839_00_0000_0000_0021_6AC3_1

Gloss annotations:
- Glosses are annotated separately for each hand
- the gloss tiers are called "GlossL" and "GlossR"
- two-handed signs are annotated identically on the tiers of both hands (this also inflates the overall count of
  annotated glosses

The translations into Dutch are in the tier called "TranslationNarrow".

Both gloss and translation tiers exist for both speakers S1 and S2. Hence, the actual tier names of interest are:
- GlossL S1
- GlossL S2
- GlossR S1
- GlossR S2
- TranslationNarrow S1
- TranslationNarrow S2

Not all ELAN files have gloss annotations or translations. The code needs to assume that any tier type could be missing.
"""


def get_elan_sentences_ngt_corpus(elan_path: str):

    eaf = pympi.Elan.Eaf(elan_path)  # TODO add "suppress_version_warning=True" when pympi 1.7 is released

    timeslots = eaf.timeslots

    for participant in ["S1", "S2"]:
        dutch_tier_name = "TranslationNarrow " + participant
        if dutch_tier_name not in eaf.tiers:
            continue

        # tiers is defined as follows (http://dopefishh.github.io/pympi/Elan.html):
        #   {tier_name -> (aligned_annotations, reference_annotations, attributes, ordinal)}
        # aligned_annotations:
        #   [{id -> (begin_ts, end_ts, value, svg_ref)}]
        # reference_annotations:
        #   [{id -> (reference, value, previous, svg_ref)}]
        #
        # - "ts" means timeslot, which references a time value in miliseconds
        # - "value" is the actual annotation content
        # - "svg_ref" is an optional reference to an SVG image that is always None in our files

        dutch_text = list(eaf.tiers[dutch_tier_name][0].values())

        # collect all glosses in the entire ELAN file

        all_glosses = []

        for hand in ["R", "L"]:
            hand_tier = "Gloss" + hand + " " + participant
            if hand_tier not in eaf.tiers:
                continue

            glosses = {}

            for gloss_id, (start, end, value, _) in eaf.tiers[hand_tier][0].items():
                glosses[gloss_id] = {"start": timeslots[start], "end": timeslots[end], "gloss": value, "hand": hand}

            all_glosses += list(glosses.values())

        for (start, end, value, _) in dutch_text:
            sentence = {"participant": participant, "start": timeslots[start], "end": timeslots[end], "dutch": value}

            # Add glosses whose timestamps are within this sentence
            glosses_in_sentence = [item for item in all_glosses if item["start"] >= sentence["start"] and item["end"] <= sentence["end"]]

            sentence["glosses"] = list(sorted(glosses_in_sentence, key=lambda d: d["start"]))

            yield sentence



# -*- coding: utf-8 -*-

from setuptools import setup, find_packages

with open("README.md", "r") as fh:
    long_description = fh.read()

packages = find_packages()
print(packages)

setup(
    name="sign-language-datasets",
    packages=packages,
    version="0.1.7",
    description="TFDS Datasets for sign language",
    author="Amit Moryossef",
    author_email="amitmoryossef@gmail.com",
    url="https://github.com/sign-language-processing/datasets",
    keywords=[],
    install_requires=["typing-extensions", "python-dotenv", "tqdm", "pose-format", "tfds-nightly", "tensorflow",
                      "numpy", "pympi-ling", "Pillow", "opencv-python==4.5.5.64"],
    tests_require=['pytest', 'pytest-cov'],
    long_description=long_description,
    long_description_content_type="text/markdown",
    classifiers=[
        "Programming Language :: Python :: 3.6",
    ],
    include_package_data=True,
)



"""WLASL: A large-scale dataset for Word-Level American Sign Language"""
import json

import tensorflow as tf
import tensorflow_datasets as tfds
from tensorflow.io.gfile import GFile
from tqdm import tqdm

from ..warning import dataset_warning
from ...datasets.config import SignDatasetConfig
from ...utils.downloaders.aslpro import download_aslpro
from ...utils.downloaders.youtube import download_youtube

_DESCRIPTION = """
A large-scale dataset for Word-Level American Sign Language
"""

_CITATION = """
@inproceedings{dataset:li2020word,
    title={Word-level Deep Sign Language Recognition from Video: A New Large-scale Dataset and Methods Comparison},
    author={Li, Dongxu and Rodriguez, Cristian and Yu, Xin and Li, Hongdong},
    booktitle={The IEEE Winter Conference on Applications of Computer Vision},
    pages={1459--1469},
    year={2020}
}
"""

_INDEX_URL = "https://raw.githubusercontent.com/dxli94/WLASL/0ac8108282aba99226e29c066cb8eab847ef62da/start_kit/WLASL_v0.3.json"

_POSE_URLS = {"openpose": "https://www.robots.ox.ac.uk/~vgg/research/bsl1k/data/info/wlasl.tar"}


class Wlasl(tfds.core.GeneratorBasedBuilder):
    """DatasetBuilder for wlasl dataset."""

    VERSION = tfds.core.Version("0.3.0")
    RELEASE_NOTES = {"0.3.0": "fix deafasl links"}

    BUILDER_CONFIGS = [SignDatasetConfig(name="default", include_video=False, include_pose="openpose")]

    def _info(self) -> tfds.core.DatasetInfo:
        """Returns the dataset metadata."""

        features = {
            "id": tfds.features.Text(),
            "signer": tfds.features.Text(),
            "gloss": tfds.features.Text(),
            "gloss_variation": tf.int32,
            "video_file": {"start": tf.int32, "end": tf.int32, "remote": tfds.features.Text(),},
            "fps": tf.int32,
            "bbox": tfds.features.BBoxFeature(),
        }

        if self._builder_config.include_video:
            features["video"] = self._builder_config.video_feature((None, None))
            features["video_file"]["local"] = tfds.features.Text()

        if self._builder_config.include_pose == "openpose":
            features["pose"] = {
                "data": tfds.features.Tensor(shape=(None, 1, 137, 2), dtype=tf.float32),
                "conf": tfds.features.Tensor(shape=(None, 1, 137), dtype=tf.float32),
            }

        return tfds.core.DatasetInfo(
            builder=self,
            description=_DESCRIPTION,
            features=tfds.features.FeaturesDict(features),
            supervised_keys=("video", "gloss"),
            homepage="https://dxli94.github.io/WLASL/",
            citation=_CITATION,
        )

    def _download_video(self, url: str, dl_manager):
        print("url", url)

        if "aslpro" in url:
            return dl_manager.download_custom(url, download_aslpro)
        elif "youtube" in url or "youtu.be" in url:
            return dl_manager.download_custom(url, download_youtube)
        else:
            return dl_manager.download(url)

    def _download_videos(self, data, dl_manager):
        videos = {}
        for datum in data:
            for instance in datum["instances"]:
                videos[instance["video_id"]] = instance["url"]

        paths = {}
        for video_id, video in tqdm(videos.items(), total=len(videos)):
            print(video_id, video)
            try:
                paths[video_id] = self._download_video(video, dl_manager)
            except (FileNotFoundError, ConnectionError) as err:
                print("Failed to download", video, str(err))

        return paths

    def _split_generators(self, dl_manager: tfds.download.DownloadManager):
        """Returns SplitGenerators."""
        dataset_warning(self)

        data_index_path = dl_manager.download(_INDEX_URL)

        # Download videos and update paths
        with GFile(data_index_path, "r") as f:
            data = json.load(f)

            if self._builder_config.include_video:
                paths = self._download_videos(data, dl_manager)
                for datum in data:
                    for instance in datum["instances"]:
                        instance["video"] = paths[instance["video_id"]] if instance["video_id"] in paths else None

        if self._builder_config.include_pose == "openpose":
            pose_path = dl_manager.download_and_extract(_POSE_URLS[self._builder_config.include_pose])
        else:
            pose_path = None

        return {
            "train": self._generate_examples(data, pose_path, "train"),
            "validation": self._generate_examples(data, pose_path, "val"),
            "test": self._generate_examples(data, pose_path, "test"),
        }

    def _generate_examples(self, data, pose_path, split):
        """Yields examples."""

        counter = 0
        for datum in data:
            counter += len(datum["instances"])
        print("counter", counter)

        # print(data)
        # print(pose_path)
        # print(len(data))

        raise Exception("die")
        # counter = 0
        # for datum in data:
        #     for instance in datum["instances"]:
        #         if instance["split"] == split and instance["video"] is not None:
        #             yield counter, {
        #                 "video": instance["video"],
        #                 "url": instance["url"],
        #                 "start": instance["frame_start"],
        #                 "end": instance["frame_end"],
        #                 "fps": instance["fps"],
        #                 "signer_id": instance["signer_id"],
        #                 "bbox": instance["bbox"],
        #                 "gloss": datum["gloss"],
        #                 "gloss_variation": instance["variation_id"],
        #             }
        #         counter += 1



"""SignBank dataset."""
import re
import xml.etree.ElementTree as ET
from datetime import datetime
from typing import List

import tensorflow as tf
import tensorflow_datasets as tfds
from tqdm import tqdm

from ..warning import dataset_warning
from ...datasets import SignDatasetConfig

_DESCRIPTION = """
SignBank Site: SignWriting Software for Sign Languages, including SignMaker 2017, 
SignPuddle Online, the SignWriting Character Viewer, SignWriting True Type Fonts, 
Delegs SignWriting Editor, SignBank Databases in FileMaker, SignWriting DocumentMaker, 
SignWriting Icon Server, the International SignWriting Alphabet (ISWA 2010) HTML Reference Guide, 
the ISWA 2010 Font Reference Library and the RAND Keyboard for SignWriting.
"""

# TODO(SignBank): BibTeX citation
_CITATION = """
"""


def is_signwriting(fsw: str) -> bool:
    return bool(re.match(r"[A]?[abcdef0-9S]*?([BLMR])(\d{3})x(\d{3})", fsw))


PUDDLES = {
    2: ["my", "mm"],  # Myanmar Dictionary",
    4: ["en", "us"],  # Dictionary US",
    5: ["en", "us"],  # Literature US",
    11: ["en", "sg"],  # Singapore Sign Language (SgSL) Dictionary",
    12: ["zh-CN", "hk"],  # Hong Kong Dictionary",
    13: ["zh-CN", "hk"],  # Hong Kong Literature",
    14: ["en", "sg"],  # Singapore Sign Language (SgSL) Literature",
    16: ["es", "hn"],  # Diccionario Honduras",
    17: ["", ""],  # Deaf Harbor",
    18: ["am", "et"],  # Dictionary Ethiopia",
    19: ["pl", "pl"],  # Słownik PL",
    20: ["fr", "ch"],  # Littérature CH-fr",
    21: ["en", "us"],  # Encyclopedia US",
    22: ["fr", "ch"],  # Encyclopédie CH-fr",
    23: ["no", "no"],  # Litteratur NO",
    24: ["no", "no"],  # Leksikon NO",
    25: ["en", "us"],  # LLCN & SignTyp",
    26: ["de", "de"],  # Literatur DE",
    27: ["de", "de"],  # Enzyklopädie DE",
    28: ["en", "us"],  # ASL Bible Dictionary",
    29: ["de", "at"],  # Wörterbuch AT",
    30: ["da", "dk"],  # Ordbog Danmark",
    31: ["mt", "mt"],  # Dictionary Malta",
    32: ["en", "ng"],  # Dictionary Nigeria",
    33: ["pt", "pt"],  # Dicionário Portugal",
    34: ["th", "th"],  # Dictionary Thailand",
    35: ["en", "isl"],  # Dictionary International",
    36: ["cs", "cz"],  # Literatura CZ",
    37: ["cs", "cz"],  # Encyklopedie CZ",
    38: ["pl", "pl"],  # Literatura PL",
    39: ["pl", "pl"],  # Encyklopedie PL",
    40: ["ar", "sa"],  # Dictionary Saudi Arabia",
    41: ["es", "ar"],  # Diccionario Argentina",
    42: ["en", "au"],  # Dictionary Australia",
    43: ["fr", "be"],  # Dictionnaire BE-fr",
    44: ["nl", "be"],  # Woordenboek Flanders",
    45: ["es", "bo"],  # Diccionario Bolivia",
    46: ["pt", "br"],  # Dicionário Brasil",
    47: ["fr", "ca"],  # Dictionnaire Quebec",
    48: ["de", "ch"],  # Wörterbuch CH-de",
    49: ["fr", "ch"],  # Dictionnaire CH-fr",
    50: ["it", "ch"],  # Dizionario CH-it",
    51: ["es", "co"],  # Diccionario Colombia",
    52: ["sk", "sk"],  # Slovník CZ",
    53: ["de", "de"],  # Wörterbuch DE",
    54: ["", ""],  # Vortaro",
    55: ["es", "es"],  # Diccionario España",
    56: ["ca", "es"],  # Diccionario Catalán",
    57: ["fi", "fi"],  # Dictionary Finland",
    58: ["fr", "fr"],  # Dictionnaire FR",
    59: ["en", "gb"],  # Dictionary Great Britain",
    60: ["en", "ie"],  # Dictionary Northern Ireland",
    61: ["gr", "gr"],  # Dictionary Greece",
    62: ["en", "ir"],  # Dictionary Ireland",
    63: ["it", "it"],  # Dizionario IT",
    64: ["ja", "jp"],  # Dictionary Japan",
    65: ["es", "mx"],  # Diccionario Mexico",
    66: ["ms", "my"],  # Dictionary Malaysia",
    67: ["es", "ni"],  # Diccionario Nicaragua",
    68: ["nl", "nl"],  # Woordenboek NL",
    69: ["", ""],  # Ordbok NO",
    70: ["en", "nz"],  # Dictionary New Zealand",
    71: ["es", "pe"],  # Diccionario Peru",
    72: ["fil", "ph"],  # Dictionary Philippines",
    73: ["sv", "se"],  # Ordbok Sverige",
    74: ["sl", "sl"],  # Slovar Slovenia",
    75: ["zh-tw", "tw"],  # Dictionary Taiwan",
    76: ["es", "ve"],  # Diccionario Venezuela",
    77: ["en", "za"],  # Dictionary South Africa",
    78: ["ko", "kr"],  # Dictionary Korea",
    79: ["sw", "ke"],  # Dictionary Kenya",
    80: ["", ""],  # Project 2 Dictionary Sorting",
    81: ["fr", "ca"],  # Littérature Quebec",
    82: ["sq", "al"],  # Dictionary Albania",
    83: ["zh-cn", "cn"],  # Dictionary China",
    84: ["ar", "eg"],  # Dictionary Egypt",
    85: ["hi", "in"],  # Dictionary India",
    86: ["ar", "jo"],  # Dictionary Jordan",
    87: ["ur", "pk"],  # Dictionary Pakistan",
    88: ["ru", "ru"],  # Dictionary Russia",
    89: ["sk", "sk"],  # Dictionary Slovakia",
    90: ["tr", "tr"],  # Dictionary Turkey",
    91: ["ar", "sa"],  # Literature Saudi Arabia",
    92: ["ar", "jo"],  # Literature Jordan",
    93: ["es", "es"],  # Literatura España",
    94: ["ca", "es"],  # Literatura Catalán",
    95: ["fr", "be"],  # Littérature BE-fr",
    96: ["de", "ch"],  # Literatur CH-de",
    98: ["nl", "be"],  # Literatuur Flanders",
    99: ["ja", "jp"],  # Literature Japan",
    100: ["am", "et"],  # Literature Ethiopia",
    103: ["mt", "mt"],  # Malta LSM Private Puddle",
    104: ["ar", "tn"],  # Dictionnaire Tunisien",
    105: ["", ""],  # DAC Private Puddle",
    106: ["ps", "af"],  # Dictionary Afghanistan",
    107: ["lt", "lt"],  # Dictionary Lithuania",
    108: ["lv", "lv"],  # Dictionary Latvia",
    109: ["et", "et"],  # Dictionary Estonia",
    110: ["he", "il"],  # Dictionary Israel",
    111: ["", ""],  # Project 1 Translate Wiki",
    112: ["es", "gt"],  # Dictionary Guatemala",
    113: ["ht", "ht"],  # Dictionary Haiti",
    114: ["pt", "br"],  # Literatura Brasil",
    115: ["pt", "pt"],  # Literatura Portugal",
    116: ["pt", "br"],  # Enciclopédia Brasil",
    117: ["pt", "pt"],  # Enciclopédia Portugal",
    118: ["da", "dk"],  # Litteratur Danmark",
    119: ["es", "ni"],  # Literatura Nicaragua",
    120: ["es", "mx"],  # Literatura Mexico",
    122: ["hu", "hu"],  # Dictionary Hungary",
    123: ["hu", "hu"],  # Literature Hungary",
    124: ["fr", "fr"],  # Literature France",
    125: ["en", "gb"],  # Literature Great Britain",
    126: ["ar", "tn"],  # Littérature Tunisien",
    127: ["mt", "mt"],  # Literature Malta",
    128: ["mw", "mw"],  # Dictionary Malawi",
    129: ["gn", "py"],  # Diccionario Paraguay",
    130: ["uk", "ua"],  # Dictionary Ukraine",
    131: ["", ""],  # Ordabók IS",
    132: ["ro", "ro"],  # Dictionary Romania",
    133: ["ne", "np"],  # Dictionary Nepal",
    134: ["bg", "bg"],  # Dictionary Bulgaria",
    135: ["es", "cl"],  # Diccionario Chile",
    136: ["es", "ec"],  # Diccionario Ecuador",
    137: ["es", "sv"],  # Diccionario El Salvador",
    138: ["ro", "ro"],  # Literature Romania",
    139: ["ro", "ro"],  # Encyclopedia Romania",
    140: ["fr", "ca"],  # Encyclopédie Quebec",
    141: ["ru", "ru"],  # Literature Russia",
    142: ["ru", "ru"],  # Encyclopedia Russia",
    143: ["es", "uy"],  # Diccionario Uruguay",
    144: ["es", "uy"],  # Literatura Uruguay",
    145: ["es", "ar"],  # Literatura Argentina",
    146: ["es", "ar"],  # Enciclopedia Argentina",
    147: ["mt", "mt"],  # Literature Malta Archive",
    148: ["sl", "sl"],  # Besedila Slovenia",
    149: ["sl", "sl"],  # Enciklopedija Slovenia",
    150: ["", ""],  # Anthropology Book Project",
    151: ["en", "us"],  # ASL Bible Books NLT",
    152: ["en", "us"],  # ASL Bible Books Shores Deaf Church",
    153: ["vn", "vn"],  # Dictionary Vietnam"
}

CACHE_BUSTER = str(datetime.today().date())

class SignBank(tfds.core.GeneratorBasedBuilder):
    """DatasetBuilder for SignBank dataset."""

    VERSION = tfds.core.Version("1.0.0")
    RELEASE_NOTES = {
        "1.0.0": "Initial release.",
    }

    BUILDER_CONFIGS = [SignDatasetConfig(name="default", include_video=False)]

    def _info(self) -> tfds.core.DatasetInfo:
        """Returns the dataset metadata."""

        features = {
            "puddle": tf.int32,
            "id": tfds.features.Text(),
            "assumed_spoken_language_code": tfds.features.Text(),
            "country_code": tfds.features.Text(),
            "created_date": tfds.features.Text(),
            "modified_date": tfds.features.Text(),
            "sign_writing": tfds.features.Sequence(tfds.features.Text()),
            "terms": tfds.features.Sequence(tfds.features.Text()),
            "user": tfds.features.Text(),
        }

        return tfds.core.DatasetInfo(
            builder=self,
            description=_DESCRIPTION,
            features=tfds.features.FeaturesDict(features),
            homepage="http://signbank.org/",
            supervised_keys=None,
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager: tfds.download.DownloadManager):
        """Returns SplitGenerators."""
        dataset_warning(self)

        index = dl_manager.download("http://signbank.org/signpuddle2.0/data/spml/")
        regex = r"\"sgn[\d]+.spml\""
        with open(index, "r", encoding="utf-8") as f:
            matches = [match.group()[1:-1] for match in re.finditer(regex, f.read(), re.MULTILINE)]
            spml_urls = ["http://signbank.org/signpuddle2.0/data/spml/" + match + "?buster=" + CACHE_BUSTER
                         for match in matches if match != "sgn0.spml"]

        spmls = dl_manager.download(spml_urls)

        return {
            "train": self._generate_examples(spmls),
        }

    def _generate_examples(self, spmls: List[str]):
        """Yields examples."""

        i = 0
        for spml in tqdm(spmls):
            tree = ET.parse(spml)
            root = tree.getroot()

            puddle = int(root.attrib["puddle"])

            children = root.iter()
            for child in children:
                if child.tag == "entry":
                    _id = child.attrib["id"]
                    mdt = int(child.attrib["mdt"]) if "mdt" in child.attrib and child.attrib["mdt"] != "" else 0
                    cdt = int(child.attrib["cdt"]) if "cdt" in child.attrib and child.attrib["cdt"] != "" else mdt
                    usr = child.attrib["usr"] if "usr" in child.attrib else ""
                    texts = [c.text for c in child.iter() if c.tag != "src" and c.text is not None]

                    signwriting_texts = []
                    spoken_texts = []
                    for text in texts:
                        if is_signwriting(text):
                            signwriting_texts.append(text)
                        else:
                            spoken_texts.append(text)

                    # print(child.attrib)
                    if len(texts) > 0:
                        sample_id = "_".join([str(puddle), _id, str(i)])
                        assumed_spoken_language_code, country_code = PUDDLES[puddle] if puddle in PUDDLES else ["", ""]
                        i += 1
                        yield sample_id, {
                            "puddle": puddle,
                            "id": _id,
                            "assumed_spoken_language_code": assumed_spoken_language_code,
                            "country_code": country_code,
                            "created_date": str(datetime.fromtimestamp(cdt)),
                            "modified_date": str(datetime.fromtimestamp(mdt)),
                            "sign_writing": signwriting_texts,
                            "terms": spoken_texts,
                            "user": usr,
                        }



"""A database of lexical and phonological properties of American Sign Language signs. """
from os import path
import tensorflow_datasets as tfds
import csv

from ..warning import dataset_warning
from ...datasets.config import SignDatasetConfig

_DESCRIPTION = """
ASL-LEX is a database of lexical and phonological properties of American Sign Language signs. 
It was first released in 2016 with nearly 1,000 signs. 
ASL-LEX was updated in Fall 2020 with greatly expanded information and an increased size of 2,723 signs.
"""

_CITATION = """
@article{caselli2017asl,
  title={ASL-LEX: A lexical database of American Sign Language},
  author={Caselli, Naomi K and Sehyr, Zed Sevcikova and Cohen-Goldberg, Ariel M and Emmorey, Karen},
  journal={Behavior research methods},
  volume={49},
  pages={784--801},
  year={2017},
  publisher={Springer}
}
"""

header_file = path.join(path.dirname(path.realpath(__file__)), "data-key.csv")


class AslLex(tfds.core.GeneratorBasedBuilder):
    """DatasetBuilder for asl-lex dataset."""

    VERSION = tfds.core.Version("2.0.0")
    RELEASE_NOTES = {
        "1.0.0": "Initial release with nearly 1,000 signs.",
        "2.0.0": "greatly expanded information and an increased size of 2,723 signs",
    }

    BUILDER_CONFIGS = [
        SignDatasetConfig(name="default", include_video=True),
        SignDatasetConfig(name="annotations", include_video=False),
    ]

    def _info(self) -> tfds.core.DatasetInfo:
        """Returns the dataset metadata."""

        # Open the CSV file
        with open(header_file, "r", encoding="utf-8") as csvfile:
            # Create a CSV reader object
            reader = csv.DictReader(csvfile)
            self.features = {row["VariableName(CSV&OSF)"]: tfds.features.Text() for row in reader}

        return tfds.core.DatasetInfo(
            builder=self,
            description=_DESCRIPTION,
            features=tfds.features.FeaturesDict(self.features),
            homepage="https://asl-lex.org/",
            supervised_keys=None,
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager: tfds.download.DownloadManager):
        dataset_warning(self)

        csv_path = dl_manager.download_and_extract("https://osf.io/download/9nygd/")

        return {"train": self._generate_examples(csv_path)}

    def _generate_examples(self, csv_path: str):
        """Yields examples."""

        with open(csv_path, "r", encoding="ISO-8859-1") as csvfile:
            # Create a CSV reader object
            reader = csv.DictReader(csvfile)

            for i, row in enumerate(reader):
                new_row = {f: row[f] for f in self.features}
                yield str(i), new_row



from .aslg_pc12 import AslgPc12
from .asl_lex import AslLex
from .autsl import AUTSL
from .chicagofswild import ChicagoFSWild
from .config import SignDatasetConfig
from .dgs_types import DgsTypes
from .dgs_corpus import DgsCorpus
from .dicta_sign import DictaSign
from .how2sign import How2Sign
from .rwth_phoenix2014_t import RWTHPhoenix2014T
from .sign2mint import Sign2MINT
from .signbank import SignBank
from .signtyp import SignTyp
from .signsuisse import SignSuisse
from .swojs_glossario import SwojsGlossario
from .wlasl import Wlasl
from .ngt_corpus import NGTCorpus
from .bsl_corpus import BslCorpus
from .wmt_slt import WMTSLT



# -*- coding: utf-8 -*-
"""dgs_document_split.ipynb

Automatically generated by Colaboratory.

Original file is located at
    https://colab.research.google.com/drive/19pHmLuIEAKFn4BqVr7cwNRaVxQHWKNI7
"""

# ! pip install sign-language-datasets==0.0.12

import json

import numpy as np

import tensorflow_datasets as tfds
import sign_language_datasets.datasets
from sign_language_datasets.datasets.config import SignDatasetConfig
from sign_language_datasets.datasets.dgs_corpus.dgs_utils import get_elan_sentences

from typing import Optional, Tuple

np.random.seed(1)

# Videos 1177918 and 1432043 have 25 fps, start and end frame won't match

INCORRECT_FRAMERATE = ["1432043", "1177918"]


def get_split_indexes(total_size: int, dev_size: int, test_size) -> Tuple[np.array, np.array, np.array]:
    train_indexes = np.arange(total_size, dtype=np.int32)

    np.random.shuffle(train_indexes)

    # high inclusive

    dev_indexes = np.random.choice(train_indexes, size=(dev_size,), replace=False)

    remaining_train_indexes = np.asarray([i for i in train_indexes if i not in dev_indexes])

    test_indexes = np.random.choice(remaining_train_indexes, size=(test_size,), replace=False)

    remaining_train_indexes = np.asarray([i for i in remaining_train_indexes if i not in test_indexes])

    return remaining_train_indexes, dev_indexes, test_indexes


config = SignDatasetConfig(name="only-annotations", version="1.0.0", include_video=False, include_pose=None)
dgs_corpus = tfds.load("dgs_corpus", builder_kwargs=dict(config=config))


def get_split(dev_size: int, test_size: int):
    ids = np.array([datum["id"].numpy().decode("utf-8") for datum in dgs_corpus["train"] if datum["id"] not in INCORRECT_FRAMERATE])

    train_indexes, dev_indexes, test_indexes = get_split_indexes(len(ids), dev_size=dev_size, test_size=test_size)

    print("Number of entire files in each split:")
    print(str({"train": len(train_indexes), "dev": len(dev_indexes), "test": len(test_indexes)}))

    return {
        "dgs_corpus_version": "3.0.0",
        "train": list(ids[train_indexes]),
        "dev": list(ids[dev_indexes]),
        "test": list(ids[test_indexes]),
    }


split = get_split(dev_size=10, test_size=10)

with open("split.json", "w") as outfile:
    json.dump(split, outfile, indent=4)

# ! cat split.json

"""## compute sentence statistics for this split"""


def get_split_name_from_id(_id: str) -> str:
    for key in split.keys():
        if _id in split[key]:
            return key

    return "none"


sentences_found = {"train": 0, "dev": 0, "test": 0, "none": 0}

for datum in dgs_corpus["train"]:

    _id = datum["id"].numpy().decode("utf-8")

    split_name = get_split_name_from_id(_id)

    elan_path = datum["paths"]["eaf"].numpy().decode("utf-8")
    sentences = get_elan_sentences(elan_path)

    for sentence in sentences:
        gloss_sequence = " ".join([s["gloss"] for s in sentence["glosses"]])
        german_sentence = sentence["german"]

        if gloss_sequence != "" and german_sentence != "":
            sentences_found[split_name] += 1

print(sentences_found)



import pympi

from lxml import etree
from typing import Dict, List


def get_elan_sentences(elan_path: str):

    eaf = pympi.Elan.Eaf(elan_path)  # TODO add "suppress_version_warning=True" when pympi 1.7 is released

    timeslots = eaf.timeslots

    for participant in ["A", "B"]:
        german_tier_name = "Deutsche_Übersetzung_" + participant
        if german_tier_name not in eaf.tiers:
            continue

        german_text = eaf.tiers[german_tier_name][0]

        english_tier_name = "Translation_into_English_" + participant
        english_text = list(eaf.tiers[english_tier_name][0].values()) if english_tier_name in eaf.tiers else []

        all_glosses = []
        for hand in ["r", "l"]:
            hand_tier = "Lexem_Gebärde_" + hand + "_" + participant
            if hand_tier not in eaf.tiers:
                continue

            gloss = {
                _id: {"start": timeslots[s], "end": timeslots[e], "gloss": val, "hand": hand}
                for _id, (s, e, val, _) in eaf.tiers[hand_tier][0].items()
            }
            for tier in ["Lexeme_Sign", "Gebärde", "Sign"]:
                items = eaf.tiers[tier + "_" + hand + "_" + participant][1]
                for ref, val, _1, _2 in items.values():
                    if ref in gloss:  # 2 files have a missing reference
                        gloss[ref][tier] = val

            all_glosses += list(gloss.values())

        all_mouthings = []

        tier_name = "Mundbild_Mundgestik_" + participant
        items = eaf.tiers[tier_name][0]

        # structure of entries:
        # {'a2768296': ('ts42', 'ts43', 'tochter', None), ... }

        for s, e, val, _ in items.values():
            mouthing_entry = {"start": timeslots[s], "end": timeslots[e], "mouthing": val}
            all_mouthings.append(mouthing_entry)

        for _id, (s, e, val, _) in german_text.items():
            sentence = {"id": _id, "participant": participant, "start": timeslots[s], "end": timeslots[e], "german": val}

            # Add English sentence
            english_sentence = [val2 for (s2, e2, val2, _) in english_text if s == s2 and e == e2]
            sentence["english"] = english_sentence[0] if len(english_sentence) > 0 else None

            # Add glosses
            sentence["glosses"] = list(
                sorted(
                    [item for item in all_glosses if item["start"] >= sentence["start"] and item["end"] <= sentence["end"]],
                    key=lambda d: d["start"],
                )
            )

            # add mouthings
            sentence["mouthings"] = list(
                sorted(
                    [item for item in all_mouthings if item["start"] >= sentence["start"] and item["end"] <= sentence["end"]],
                    key=lambda d: d["start"],
                )
            )

            yield sentence


def get_child_elements(root: etree.ElementTree,
                       element_name: str,
                       attributes_to_extract: List[str]) -> Dict[str, Dict[str, str]]:
    """

    :param root:
    :param element_name:
    :param attributes_to_extract:
    :return:
    """

    elements = root.xpath("/ilex-data/" + element_name)  # type: List[etree.Element]

    by_id = {}

    for element in elements:
        id_ = element.get("id")
        by_id[id_] = {}
        for attribute_name in attributes_to_extract:
            value = element.get(attribute_name)
            by_id[id_][attribute_name] = value

    return by_id


def get_signer_ids_from_ilex(ilex_path: str) -> Dict[str, List[str]]:
    """

    File structure:

    <ilex-data source="meinedgs.de" version="1.1" database_version="51">
        <camera_perspective id="1" code="A1" english="Frontal view on informant A"
            localised="Frontalansicht Informant A" visible_persons="{1}"/>
        <camera_perspective id="2" code="B1" english="Frontal view on informant B"
            localised="Frontalansicht Informant B" visible_persons="{2}"/>
        <camera_perspective id="3" code="C" english="Total on all three persons"
            localised="Totale auf alle drei Personen" visible_persons="{2,3,1}"/>
        <movie_track id="3" movie="1" camera_perspective="3" path="./1177918_1c.mp4"
            track_length="00:09:25:04"/>
        <movie_track id="1" movie="1" camera_perspective="1" path="./1177918_1a1.mp4"
            track_length="00:09:25:04"/>
        <movie_track id="2" movie="1" camera_perspective="2" path="./1177918_1b1.mp4"
            track_length="00:09:25:04"/>
        <informant id="1" sex="1" name="SH-12" short_name="SH-12"/>
        <informant id="2" sex="1" name="SH-13" short_name="SH-13"/>
        <informant id="3" sex="2" name="sh-mod-1" short_name="sh-mod-1"/>
        <participation id="1" movie="1" role="1" informant="1"/>
        <participation id="2" movie="1" role="1" informant="2"/>
        <participation id="3" movie="1" role="2" informant="3"/>
        <!--...-->
    </ilex-data>

    :param ilex_path:
    :return:
    """

    root = etree.parse(ilex_path)

    informant_dict = get_child_elements(root=root,
                                        element_name="informant",
                                        attributes_to_extract=["name"])

    camera_perspective_dict = get_child_elements(root=root,
                                                 element_name="camera_perspective",
                                                 attributes_to_extract=["visible_persons", "code"])

    signer_identities_by_perspective = {}  # type: Dict[str, List[str]]

    for camera_perspective in camera_perspective_dict.values():

        # extract A, B or C without trailing numbers

        clean_code = camera_perspective["code"][0].lower()

        # remove enclosing "{" and "}" for list of informant ids

        ids_of_visible_persons = camera_perspective["visible_persons"][1:-1].split(",")

        names_of_visible_persons = [informant_dict[id_]["name"] for id_ in ids_of_visible_persons]

        signer_identities_by_perspective[clean_code] = names_of_visible_persons

    return signer_identities_by_perspective



"""aslg_pc12 dataset."""

import tensorflow_datasets as tfds

from . import aslg_pc12


class AslgPc12Test(tfds.testing.DatasetBuilderTestCase):
    """Tests for aslg_pc12 dataset."""

    DATASET_CLASS = aslg_pc12.AslgPc12
    SPLITS = {
        "train": 10,  # Number of fake train example
    }
    DL_DOWNLOAD_RESULT = ["sample-corpus-asl-en.asl", "sample-corpus-asl-en.en"]


if __name__ == "__main__":
    tfds.testing.test_main()



"""How2Sign: A multimodal and multiview continuous American Sign Language (ASL) dataset"""
import os
from itertools import chain
from os import path

import tensorflow as tf
import tensorflow_datasets as tfds
from pose_format.utils.openpose import load_openpose_directory

from ..warning import dataset_warning
from ...datasets.config import SignDatasetConfig
from ...utils.features import PoseFeature

_DESCRIPTION = """
A multimodal and multiview continuous American Sign Language (ASL) dataset, 
consisting of a parallel corpus of more than 80 hours of sign language videos and a set of corresponding modalities 
including speech, English transcripts, and depth.
"""

_CITATION = """
@inproceedings{Duarte_CVPR2021,
    title={{How2Sign: A Large-scale Multimodal Dataset for Continuous American Sign Language}},
    author={Duarte, Amanda and Palaskar, Shruti and Ventura, Lucas and Ghadiyaram, Deepti and DeHaan, Kenneth and
                   Metze, Florian and Torres, Jordi and Giro-i-Nieto, Xavier},
    booktitle={Conference on Computer Vision and Pattern Recognition (CVPR)},
    year={2021}
}
"""

_SPLITS = {
    tfds.Split.TRAIN: {
        "rgb_clips_front": "https://drive.google.com/uc?id=1dYey1F_SeHets-UO8F9cE3VMhRBO-6e0&export=download",
        "rgb_clips_side": "https://drive.google.com/uc?id=1PIYIIOxR2vnUDzSHdq3uyoUCoIJvsuNW&export=download",
        "bfh_2d_front": "https://drive.google.com/uc?id=1lnsDN-LxcsroOmetdG5_sXYXZ7setlS4&export=download",
        "bfh_2d_side": None,
        "translation": None,
    },
    tfds.Split.VALIDATION: {
        "rgb_clips_front": "https://drive.google.com/uc?id=1oVZyTWhHShyqshC2kUrfWnBF8apIR7Z1&export=download",
        "rgb_clips_side": "https://drive.google.com/uc?id=1vJVV777_bmSeA2_k7iGdZu2izooeKUrq&export=download",
        "bfh_2d_front": "https://drive.google.com/uc?id=1aOhRknNWj8APdxHmwJdQrMo5xuIGNXxM&export=download",
        "bfh_2d_side": None,
        "translation": None,
    },
    tfds.Split.TEST: {
        "rgb_clips_front": "https://drive.google.com/uc?id=1d6GHqu0_8IGiKbu3sTZHtMb0DGhbHSMu&export=download",
        "rgb_clips_side": "https://drive.google.com/uc?id=1gKV_TloCbMyMhOdYvr_a-6I-PTf0Sjyi&export=download",
        "bfh_2d_front": "https://drive.google.com/uc?id=1quj8Ipm56pH65KAKK3Pc-sqZ0ozw2gSe&export=download",
        "bfh_2d_side": None,
        "translation": None,
    },
}

_POSE_HEADERS = {"openpose": path.join(path.dirname(path.realpath(__file__)), "openpose.header")}


class How2Sign(tfds.core.GeneratorBasedBuilder):
    """DatasetBuilder for how2sign dataset."""

    VERSION = tfds.core.Version("1.0.0")
    RELEASE_NOTES = {"1.0.0": "Initial release."}

    BUILDER_CONFIGS = [SignDatasetConfig(name="default", include_video=True, include_pose="openpose")]

    def _info(self) -> tfds.core.DatasetInfo:
        """Returns the dataset metadata."""

        features = {"id": tfds.features.Text(), "fps": tf.int32}

        if self._builder_config.include_video:
            features["video"] = {
                "front": self._builder_config.video_feature((1280, 720)),
                "side": self._builder_config.video_feature((1280, 720)),
            }

        if self._builder_config.include_pose == "openpose":
            pose_header_path = _POSE_HEADERS[self._builder_config.include_pose]
            stride = 1 if self._builder_config.fps is None else 24 / self._builder_config.fps
            features["pose"] = {
                "front": PoseFeature(shape=(None, 1, 137, 2), header_path=pose_header_path, stride=stride),
                # "side": PoseFeature(shape=(None, 1, 137, 2), header_path=pose_header_path, stride=stride),
            }

        return tfds.core.DatasetInfo(
            builder=self,
            description=_DESCRIPTION,
            features=tfds.features.FeaturesDict(features),
            homepage="https://how2sign.github.io/",
            supervised_keys=None,
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager: tfds.download.DownloadManager):
        """Returns SplitGenerators."""
        dataset_warning(self)

        # Define what files are required to download
        download_keys = []
        if self._builder_config.include_video is not None:
            download_keys += ["rgb_clips_front", "rgb_clips_side"]
        if self._builder_config.include_pose is not None:
            download_keys += ["bfh_2d_front", "bfh_2d_side"]

        urls = chain.from_iterable([[split[k] for k in download_keys] for split in _SPLITS.values()])
        urls = [url for url in urls if url is not None]

        downloads = dl_manager.download_and_extract(urls)
        url_map = {u: d for u, d in zip(urls, downloads)}  # Map local paths

        return [
            tfds.core.SplitGenerator(name=name, gen_kwargs={k: url_map[v] if v is not None else None for k, v in split.items()},)
            for name, split in _SPLITS.items()
        ]

    def _generate_examples(self, rgb_clips_front: str, rgb_clips_side: str, bfh_2d_front: str, bfh_2d_side: str, translation: str):
        """ Yields examples. """

        # TODO get ids from translation file
        ids = []
        ids = [p[: -len("-rgb_front.mp4")] for p in os.listdir(path.join(rgb_clips_front, "raw_videos"))]
        ids = ids[:10]

        for _id in ids:
            datum = {
                "id": _id,
                "fps": 24,
            }

            if self.builder_config.include_video:
                datum["video"] = {
                    "front": path.join(rgb_clips_front, "raw_videos", _id + "-rgb_front.mp4"),
                    "side": path.join(rgb_clips_side, "raw_videos", _id + "-rgb_side.mp4"),
                }

            if self._builder_config.include_pose == "openpose":
                front_path = path.join(bfh_2d_front, "openpose_output", "json", _id + "-rgb_front")
                front_pose = load_openpose_directory(front_path, fps=24, width=1280, height=720)

                # TODO add side pose when available
                # side_path = path.join(bfh_2d_side, 'openpose_output', 'json', _id + '-rgb_side')
                # side_pose = load_openpose_directory(side_path, fps=24, width=1280, height=720)

                datum["pose"] = {"front": front_pose}

            yield _id, datum



"""AUTSL: Large Scale Signer Independent Isolated SLR Dataset (Turkish Sign Language)"""
import csv
import os
from os import path
from typing import Union, Dict
from zipfile import ZipFile

import tensorflow as tf
import tensorflow_datasets as tfds
from tensorflow.io.gfile import GFile
from tensorflow_datasets.core.download.resource import get_dl_dirname
from tqdm import tqdm

from ..warning import dataset_warning
from ...datasets.config import SignDatasetConfig
from ...utils.features import PoseFeature

_DESCRIPTION = """
A large-scale, multimodal dataset that contains isolated Turkish sign videos.
It contains 226 signs that are performed by 43 different signers. There are 36,302 video samples in total.
It contains 20 different backgrounds with several challenges.
"""

_CITATION = """
@article{sincan2020autsl,
  title={AUTSL: A Large Scale Multi-Modal Turkish Sign Language Dataset and Baseline Methods},
  author={Sincan, Ozge Mercanoglu and Keles, Hacer Yalim},
  journal={IEEE Access},
  volume={8},
  pages={181340--181355},
  year={2020},
  publisher={IEEE}
}
"""

_OPENPOSE_DISCLAIMER = """
OpenPose skeleton for AUTSL was processed using the Single-Network Whole-Body Pose Estimation Model (BODY_135).
https://github.com/CMU-Perceptual-Computing-Lab/openpose_train/blob/master/experimental_models/README.md#single-network-whole-body-pose-estimation-model

In cases where multiple skeletons were detected in a frame, the most centered skeleton is used.
For more information, contact Necati Cihan Camgöz (cihancamgoz@gmail.com)"""

_TRAIN_VIDEOS = "http://158.109.8.102/AuTSL/data/train/train_set_vfbha39.zip"  # 18 files
_TRAIN_LABELS = "http://158.109.8.102/AuTSL/data/train/train_labels.csv"

_VALID_VIDEOS = "http://158.109.8.102/AuTSL/data/validation/val_set_bjhfy68.zip"  # 3 files
_VALID_LABELS = "https://nlp.biu.ac.il/~amit/datasets/public/autsl_validation_labels.csv"

_TEST_VIDEOS = "http://158.109.8.102/AuTSL/data/test/test_set_xsaft57.zip"  # 3 files
_TEST_LABELS = "https://nlp.biu.ac.il/~amit/datasets/public/autsl_test_labels.csv"

_CLASSES = "https://data.chalearnlap.cvc.uab.cat/AuTSL/data/SignList_ClassId_TR_EN.csv"

_POSE_URLS = {
    "holistic": "https://nlp.biu.ac.il/~amit/datasets/poses/holistic/autsl.tar.gz",
    "openpose": "https://nlp.biu.ac.il/~amit/datasets/poses/openpose/autsl.tar.gz",
}
_POSE_HEADERS = {
    "holistic": path.join(path.dirname(path.realpath(__file__)), "holistic.poseheader"),
    "openpose": path.join(path.dirname(path.realpath(__file__)), "openpose_135.poseheader"),
}


class AUTSL(tfds.core.GeneratorBasedBuilder):
    """DatasetBuilder for AUTSL dataset."""

    VERSION = tfds.core.Version("1.0.0")
    RELEASE_NOTES = {"1.0.0": "Initial release."}

    BUILDER_CONFIGS = [
        SignDatasetConfig(name="default", include_video=True, include_pose=None),
        SignDatasetConfig(name="holistic", include_video=False, include_pose="holistic"),
        SignDatasetConfig(name="openpose", include_video=False, include_pose="openpose"),
    ]

    def __init__(self, **kwargs):
        super(AUTSL, self).__init__(**kwargs)

        self.train_decryption_key = "MdG3z6Eh1t"  # publically available in http://chalearnlap.cvc.uab.es/dataset/40/data/66/description/
        self.valid_decryption_key = "bhRY5B9zS2"  # publically available in http://chalearnlap.cvc.uab.es/dataset/40/data/65/description/
        self.test_decryption_key = "ds6Kvdus3o"  # publically available in http://chalearnlap.cvc.uab.es/dataset/40/data/67/description/

    def _info(self) -> tfds.core.DatasetInfo:
        """Returns the dataset metadata."""

        features = {
            "id": tfds.features.Text(),
            "signer": tf.int32,
            "sample": tf.int32,
            "gloss_id": tf.int32,
            "meaning": {
                "english": tfds.features.Text(),
                "turkish": tfds.features.Text()
            }
        }

        if self._builder_config.include_video:
            features["fps"] = tf.int32
            features["video"] = self._builder_config.video_feature((512, 512))
            features["depth_video"] = self._builder_config.video_feature((512, 512), 1)

        # Add poses if requested
        if self._builder_config.include_pose is not None:
            pose_header_path = _POSE_HEADERS[self._builder_config.include_pose]
            stride = 1 if self._builder_config.fps is None else 30 / self._builder_config.fps

            if self._builder_config.include_pose == "openpose":
                pose_shape = (None, 1, 135, 2)
            if self._builder_config.include_pose == "holistic":
                pose_shape = (None, 1, 543, 4)

            features["pose"] = PoseFeature(shape=pose_shape, header_path=pose_header_path, stride=stride)

        return tfds.core.DatasetInfo(
            builder=self,
            description=_DESCRIPTION,
            features=tfds.features.FeaturesDict(features),
            supervised_keys=("video", "gloss_id"),
            homepage="http://chalearnlap.cvc.uab.es/dataset/40/description/",
            citation=_CITATION,
        )

    def _download_and_extract_multipart(self, dl_manager: tfds.download.DownloadManager, url: str, parts: int,
                                        pwd: str = None):
        """Download and extract multipart zip file"""

        # Write OpenPose disclaimer
        if self._builder_config.include_pose == "openpose":
            print(_OPENPOSE_DISCLAIMER)

        # Make sure not already downloaded
        dirname = get_dl_dirname(url)
        output_path = os.path.join(dl_manager._download_dir, dirname)
        output_path_extracted = os.path.join(dl_manager._extract_dir, dirname)

        print("output_path", output_path)
        print("output_path_extracted", output_path_extracted)

        if not os.path.isfile(output_path):
            parts = [url + f".{i + 1:03}" for i in range(parts)]
            files = dl_manager.download(parts)

            # Cat parts to single file
            with open(output_path, "ab") as cat_file:
                for f in files:
                    with open(f, "rb") as z:
                        cat_file.write(z.read())

        if not os.path.isdir(output_path_extracted):
            # Extract file
            os.makedirs(output_path_extracted)

            pwd_bytes = bytes(pwd, "utf-8") if pwd is not None else None

            with ZipFile(output_path, "r") as zip_obj:
                # Loop over each file
                for file in tqdm(iterable=zip_obj.namelist(), total=len(zip_obj.namelist())):
                    zip_obj.extract(member=file, path=output_path_extracted, pwd=pwd_bytes)

        return output_path_extracted

    def load_class_labels(self, dl_manager: tfds.download.DownloadManager):
        csv_file_path = dl_manager.download(_CLASSES)

        dictionary = {}
        with open(csv_file_path, mode='r', encoding='utf-8') as csv_file:
            csv_reader = csv.DictReader(csv_file)
            for row in csv_reader:
                dictionary[int(row['ClassId'])] = {'english': row['EN'], 'turkish': row['TR']}

        return dictionary

    def _split_generators(self, dl_manager: tfds.download.DownloadManager):
        """Returns SplitGenerators."""
        dataset_warning(self)

        # Download labels
        train_labels = dl_manager.download(_TRAIN_LABELS)
        valid_labels = dl_manager.download(_VALID_LABELS)
        test_labels = dl_manager.download(_TEST_LABELS)

        # Load videos if needed
        if self._builder_config.include_video:
            train_parts = self._download_and_extract_multipart(dl_manager, url=_TRAIN_VIDEOS, parts=18,
                                                               pwd=self.train_decryption_key)
            train_videos = os.path.join(train_parts, "train")

            valid_parts = self._download_and_extract_multipart(dl_manager, url=_VALID_VIDEOS, parts=3,
                                                               pwd=self.valid_decryption_key)
            valid_videos = os.path.join(valid_parts, "val")

            test_parts = self._download_and_extract_multipart(dl_manager, url=_TEST_VIDEOS, parts=3,
                                                              pwd=self.test_decryption_key)
            test_videos = os.path.join(test_parts, "test")
        else:
            train_videos = valid_videos = test_videos = None

        # Load poses if needed

        if self._builder_config.include_pose is not None:
            pose_path = dl_manager.download_and_extract(_POSE_URLS[self._builder_config.include_pose])
            train_pose_path = path.join(pose_path, self._builder_config.include_pose, "train")
            valid_pose_path = path.join(pose_path, self._builder_config.include_pose, "validation")
            test_pose_path = path.join(pose_path, self._builder_config.include_pose, "test")
        else:
            train_pose_path = valid_pose_path = test_pose_path = None

        class_labels = self.load_class_labels(dl_manager)

        splits = [
            tfds.core.SplitGenerator(
                name=tfds.Split.TEST,
                gen_kwargs={"videos_path": test_videos, "poses_path": test_pose_path, "labels_path": test_labels,
                            "class_labels": class_labels},
            ),
            tfds.core.SplitGenerator(
                name=tfds.Split.TRAIN,
                gen_kwargs={"videos_path": train_videos, "poses_path": train_pose_path, "labels_path": train_labels,
                            "class_labels": class_labels},
            ),
            tfds.core.SplitGenerator(
                name=tfds.Split.VALIDATION,
                gen_kwargs={"videos_path": valid_videos, "poses_path": valid_pose_path, "labels_path": valid_labels,
                            "class_labels": class_labels},
            ),
        ]

        return splits

    def _generate_examples(self, videos_path: Union[str, None], poses_path: Union[str, None],
                           labels_path: Union[str, None], class_labels: Dict[int, Dict[str, str]]):
        """Yields examples."""

        if labels_path is not None:
            with GFile(labels_path, "r") as labels_file:
                labels = {sample_id: int(label_id) for sample_id, label_id in csv.reader(labels_file)}
        else:
            labels = None

        if videos_path is not None:
            samples = {tuple(f.split("_")[:2]) for f in os.listdir(videos_path)}
        elif poses_path is not None:
            samples = {tuple(f.split(".")[0].split("_")) for f in os.listdir(poses_path)}
        elif labels_path is not None:
            samples = {tuple(k.split("_")) for k in labels.keys()}
        else:
            raise Exception("Found no samples to generate")

        for signer, sample in samples:
            datum = dict({"id": signer + "_" + sample, "signer": int(signer[6:]), "sample": int(sample[6:])})
            datum["gloss_id"] = labels[datum["id"]] if labels is not None else -1

            datum["meaning"] = class_labels[datum["gloss_id"]]

            if videos_path is not None:
                datum["fps"] = self._builder_config.fps if self._builder_config.fps is not None else 30
                datum["video"] = os.path.join(videos_path, datum["id"] + "_color.mp4")
                datum["depth_video"] = os.path.join(videos_path, datum["id"] + "_depth.mp4")

            if poses_path is not None:
                datum["pose"] = os.path.join(poses_path, datum["id"] + ".pose")

            yield datum["id"], datum
